@@ -1350,12 +1350,14 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
   request_rec   *r;
   Attr          *attr;
   char          *new_hidden_tag = NULL;
+  char          *attr_method = NULL;
+  char          *attr_action = NULL;
+  char          *attr_utn    = NULL;
 
   chtml30 = GET_CHTML30(pdoc);
   doc     = chtml30->doc;
   r       = doc->r;
 
-  W_L("<form");
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
@@ -1368,34 +1370,46 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
-      value = chxj_encoding_parameter(r, value);
-      value = chxj_add_cookie_parameter(r, value, chtml30->cookie);
-      char *q;
-      q = strchr(value, '?');
-      if (q) {
-        new_hidden_tag = chxj_form_action_to_hidden_tag(doc->pool, value, 0);
-        *q = 0;
-      }
-      W_L(" action=\"");
-      W_V(value);
-      W_L("\"");
+      attr_action = chxj_encoding_parameter(r, value);
+      attr_action = chxj_add_cookie_parameter(r, attr_action, chtml30->cookie);
     }
     else if (STRCASEEQ('m','M',"method", name)) {
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
-      W_L(" method=\"");
-      W_V(value);
-      W_L("\"");
+      attr_method = apr_pstrdup(doc->pool, value);
     }
     else if (STRCASEEQ('u','U',"utn", name)) {
       /*----------------------------------------------------------------------*/
       /* CHTML 3.0                                                            */
       /* It is special only for CHTML.                                        */
       /*----------------------------------------------------------------------*/
-      W_L(" utn");
+      attr_utn = value;
     }
   }
+
+  int post_flag = (attr_method && strcasecmp(attr_method, "post") == 0) ? 1 : 0;
+
+  W_L("<form");
+  if (attr_action) {
+    char *q;
+    q = strchr(attr_action, '?');
+    if (q) {
+      new_hidden_tag = chxj_form_action_to_hidden_tag(doc->pool, attr_action, 0, post_flag);
+      *q = 0;
+    }
+    W_L(" action=\"");
+    W_V(attr_action);
+    W_L("\"");
+  }
+  if (attr_method) {
+    W_L(" method=\"");
+    W_V(attr_method);
+    W_L("\"");
+  }
+  if (attr_utn) {
+    W_L(" utn");
+  }
   W_L(">");
   if (new_hidden_tag) {
     W_V(new_hidden_tag);
