@@ -49,7 +49,9 @@ static int valid_path(request_rec *r, const char *value);
 static int valid_expires(request_rec *r, const char *value);
 static int valid_secure(request_rec *r, const char *value);
 static int check_valid_cookie_attribute(request_rec *r, const char *pair);
+static int check_valid_cookie_attribute_expires_only(request_rec *r, const char *value);
 
+apr_proc_mutex_t *global_cookie_mutex;
 
 static char *
 alloc_cookie_id(request_rec *r)
@@ -107,14 +109,14 @@ chxj_save_cookie(request_rec* r)
   apr_table_t*        new_cookie_table;
   int                 has_cookie = 0;
   cookie_t*           cookie;
-  cookie_t*           old_cookie;
+  cookie_t*           old_cookie = NULL;
   char*               refer_string;
   apr_uri_t           parsed_uri;
   int                 has_refer;
   apr_pool_t          *pool;
 
 
-  DBG(r, "start chxj_save_cookie()");
+  DBG(r, "REQ[%X] start chxj_save_cookie()", (apr_size_t)r);
 
   apr_pool_create(&pool, r->pool);
   cookie = (cookie_t*)apr_palloc(pool, sizeof(cookie_t));
@@ -126,11 +128,11 @@ chxj_save_cookie(request_rec* r)
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
-    DBG(r, "end chxj_save_cookie() no pattern");
+    DBG(r, "REQ[%X] end chxj_save_cookie() no pattern", (apr_size_t)r);
     return NULL;
   }
   if (! (entryp->action & CONVRULE_COOKIE_ON_BIT)) {
-    DBG(r, "end chxj_save_cookie() CookieOff");
+    DBG(r, "REQ[%X] end chxj_save_cookie() CookieOff", (apr_size_t)r);
     return NULL;
   }
 
@@ -143,50 +145,47 @@ chxj_save_cookie(request_rec* r)
 
   for (ii=0; ii<headers->nelts; ii++) {
     if (strcasecmp(hentryp[ii].key, "Set-Cookie") == 0) {
-      DBG(r, "=====================================");
-      DBG(r, "cookie=[%s:%s]", hentryp[ii].key, hentryp[ii].val);
+      DBG(r, "REQ[%X] cookie=[%s:%s]", (apr_size_t)r, hentryp[ii].key, hentryp[ii].val);
 
       char* key;
       char* val;
       char* buff;
 
-
-      buff = apr_pstrdup(pool, hentryp[ii].val);
-      val = strchr(buff, '=');
-      if (val) {
-        key = buff;
-        *val++ = 0;
-        apr_table_add(new_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
-        if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
-
+      char *pair = apr_psprintf(pool, "%s:%s", hentryp[ii].key, hentryp[ii].val);
+      if (check_valid_cookie_attribute_expires_only(r, pair)) {
+        buff = apr_pstrdup(pool, hentryp[ii].val);
+        val = strchr(buff, '=');
+        if (val) {
+          key = buff;
+          *val++ = 0;
+          apr_table_set(new_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
+          if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
+        }
+        has_cookie = 1;
       }
-
-      has_cookie = 1;
-      DBG(r, "=====================================");
     }
   }
   for (ii=0; ii<err_headers->nelts; ii++) {
     if (strcasecmp(err_hentryp[ii].key, "Set-Cookie") == 0) {
-      DBG(r, "=====================================");
-      DBG(r, "cookie=[%s:%s]", err_hentryp[ii].key, err_hentryp[ii].val);
+      DBG(r, "REQ[%X] err_cookie=[%s:%s]", (apr_size_t)r, err_hentryp[ii].key, err_hentryp[ii].val);
 
       char* key;
       char* val;
       char* buff;
 
 
-      buff = apr_pstrdup(pool, err_hentryp[ii].val);
-      val = strchr(buff, '=');
-      if (val) {
-        key = buff;
-        *val++ = 0;
-        apr_table_add(new_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
-        if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
-
+      char *pair = apr_psprintf(pool, "%s:%s", hentryp[ii].key, hentryp[ii].val);
+      if (check_valid_cookie_attribute_expires_only(r, pair)) {
+        buff = apr_pstrdup(pool, err_hentryp[ii].val);
+        val = strchr(buff, '=');
+        if (val) {
+          key = buff;
+          *val++ = 0;
+          apr_table_set(new_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
+          if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
+        }
+        has_cookie = 1;
       }
-
-      has_cookie = 1;
-      DBG(r, "=====================================");
     }
   }
   apr_table_unset(r->headers_out, "Set-Cookie");
@@ -206,7 +205,7 @@ chxj_save_cookie(request_rec* r)
       refer_string = apr_pstrcat(pool, refer_string, "?", r->args, NULL);
     }
     apr_table_setn(new_cookie_table, REFERER_COOKIE_KEY, apr_pstrdup(pool, refer_string));
-    DBG(r, "ADD REFER[%s]", refer_string);
+    DBG(r, "REQ[%X] ADD REFER[%s]", (apr_size_t)r, refer_string);
     has_cookie++;
   }
 
@@ -222,18 +221,18 @@ chxj_save_cookie(request_rec* r)
       for (ii=0; ii<old_cookie->cookie_headers->nelts; ii++) {
         if (hentryp && apr_table_get(new_cookie_table, hentryp[ii].key) == NULL) {
           apr_table_add(new_cookie_table, apr_pstrdup(pool, hentryp[ii].key), apr_pstrdup(pool, hentryp[ii].val));
-          has_cookie = 1;
         }
       }
-      chxj_delete_cookie(r,        old_cookie_id);
-      chxj_delete_cookie_expire(r, old_cookie_id);
+      if (has_cookie) {
+        chxj_delete_cookie(r,        old_cookie_id);
+        chxj_delete_cookie_expire(r, old_cookie_id);
+      }
     }
   }
 
   if (! has_cookie) {
-    DBG(r, "no cookie");
-    DBG(r, "end chxj_save_cookie()");
-    return NULL;
+    DBG(r, "REQ[%X] end chxj_save_cookie() (no cookie)",(apr_size_t)r);
+    return old_cookie;
   }
 
   /*
@@ -265,21 +264,25 @@ chxj_save_cookie(request_rec* r)
   }
 
   if (old_cookie_id && IS_COOKIE_LAZY(dconf)) {
-    DBG(r, "LAZY COOKIE save");
+    DBG(r, "REQ[%X] LAZY COOKIE save",(apr_size_t)r);
+    cookie->cookie_id = apr_pstrdup(r->pool, old_cookie_id);
+  }
+  else if (old_cookie_id && apr_table_get(r->headers_in, "X-Chxj-Cookie-No-Update")) {
+    DBG(r, "REQ[%X] NO UPDATE MODE",(apr_size_t)r);
     cookie->cookie_id = apr_pstrdup(r->pool, old_cookie_id);
   }
   else {
-    DBG(r, "NO LAZY COOKIE save");
+    DBG(r, "REQ[%X] NO LAZY COOKIE save",(apr_size_t)r);
     cookie->cookie_id = alloc_cookie_id(r);
   }
 
-DBG(r, "TYPE:[%d]", dconf->cookie_store_type);
+DBG(r, "REQ[%X] TYPE:[%d]", (apr_size_t)r, dconf->cookie_store_type);
   {
     int done_proc = 0;
 #if defined(USE_MYSQL_COOKIE)
     if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
       if (! chxj_save_cookie_mysql(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "faild: chxj_save_cookie_mysql() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] faild: chxj_save_cookie_mysql() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
       done_proc = 1;
@@ -288,7 +291,7 @@ DBG(r, "TYPE:[%d]", dconf->cookie_store_type);
 #if defined(USE_MEMCACHE_COOKIE)
     if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
       if (! chxj_save_cookie_memcache(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "failed: chxj_save_cookie_memcache() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_save_cookie_memcache() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
       done_proc = 1;
@@ -296,7 +299,7 @@ DBG(r, "TYPE:[%d]", dconf->cookie_store_type);
 #endif
     if (IS_COOKIE_STORE_DBM(dconf->cookie_store_type) || ! done_proc) {
       if (! chxj_save_cookie_dbm(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "failed: chxj_save_cookie_dbm() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_save_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
     }
@@ -308,7 +311,7 @@ DBG(r, "TYPE:[%d]", dconf->cookie_store_type);
 
 
 on_error:
-  DBG(r, "end chxj_save_cookie()");
+  DBG(r, "REQ[%X] end chxj_save_cookie()", (apr_size_t)r);
   return cookie;
 }
 
@@ -327,9 +330,9 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
   cookie_t*           cookie;
 
 
-  DBG(r, "start chxj_update_cookie()");
+  DBG(r, "REQ[%X] start chxj_update_cookie()",(apr_size_t)r);
   if (!old_cookie || ! old_cookie->cookie_headers || ! old_cookie->cookie_id) {
-    DBG(r, "end chxj_update_cookie() (old_cookie is null)");
+    DBG(r, "REQ[%X] end chxj_update_cookie() (old_cookie is null)", (apr_size_t)r);
     return  NULL;
   }
 
@@ -340,11 +343,11 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
-    DBG(r, "end chxj_update_cookie() no pattern");
+    DBG(r, "REQ[%X] end chxj_update_cookie() no pattern", (apr_size_t)r);
     return NULL;
   }
   if (! (entryp->action & CONVRULE_COOKIE_ON_BIT)) {
-    DBG(r, "end chxj_update_cookie() CookieOff");
+    DBG(r, "REQ[%X] end chxj_update_cookie() CookieOff", (apr_size_t)r);
     return NULL;
   }
 
@@ -357,11 +360,11 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
   chxj_delete_cookie_expire(r, old_cookie->cookie_id);
 
   if (IS_COOKIE_LAZY(dconf)) {
-    DBG(r, "LAZY MODE");
+    DBG(r, "REQ[%X] LAZY MODE",(apr_size_t)r);
     cookie->cookie_id = apr_pstrdup(r->pool, old_cookie->cookie_id);
   }
   else {
-    DBG(r, "NO LAZY MODE");
+    DBG(r, "REQ[%X] NO LAZY MODE",(apr_size_t)r);
     cookie->cookie_id = alloc_cookie_id(r);
   }
 
@@ -376,7 +379,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
                                "\n",
                                NULL);
 
-    DBG(r, "OLD COOKIE VALUE=[%s][%s]", hentryp[ii].key, hentryp[ii].val);
+    DBG(r, "REQ[%X] OLD COOKIE VALUE=[%s][%s]", (apr_size_t)r, hentryp[ii].key, hentryp[ii].val);
     store_string = apr_pstrcat(r->pool, 
                                store_string, 
                                hentryp[ii].key, 
@@ -390,7 +393,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
 #if defined(USE_MYSQL_COOKIE)
     if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
       if (!chxj_update_cookie_mysql(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "failed: chxj_update_cookie_mysql() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_update_cookie_mysql() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
       done_proc = 1;
@@ -400,7 +403,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
 #if defined(USE_MEMCACHE_COOKIE)
     if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
       if (! chxj_update_cookie_memcache(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "failed: chxj_update_cookie_memcache() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_update_cookie_memcache() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
       done_proc = 1;
@@ -408,7 +411,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
 #endif
     if (!done_proc || IS_COOKIE_STORE_DBM(dconf->cookie_store_type)) {
       if (! chxj_update_cookie_dbm(r, dconf, cookie->cookie_id, store_string)) {
-        ERR(r, "failed: chxj_update_cookie_dbm() cookie_id:[%s]", cookie->cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_update_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie->cookie_id);
         goto on_error;
       }
     }
@@ -420,7 +423,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
 
 
 on_error:
-  DBG(r, "end   chxj_update_cookie()");
+  DBG(r, "REQ[%X] end   chxj_update_cookie()", (apr_size_t)r);
   return cookie;
 }
 
@@ -444,11 +447,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
   char                    *header_cookie;
   apr_pool_t              *pool;
 
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "start chxj_load_cookie() cookie_id=[%s]", cookie_id);
+  DBG(r, "REQ[%X] start chxj_load_cookie() cookie_id=[%s]", (apr_size_t)r, cookie_id);
   chxj_cookie_expire_gc(r);
 
   apr_pool_create(&pool, r->pool);
@@ -460,11 +459,11 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
-    DBG(r, "end chxj_load_cookie() no pattern");
+    DBG(r, "REQ[%X] end chxj_load_cookie() no pattern",(apr_size_t)r);
     goto on_error0;
   }
   if (! (entryp->action & CONVRULE_COOKIE_ON_BIT)) {
-    DBG(r, "end chxj_load_cookie() CookieOff");
+    DBG(r, "REQ[%X] end chxj_load_cookie() CookieOff",(apr_size_t)r);
     goto on_error0;
   }
   load_cookie_table = apr_table_make(pool, 1);
@@ -474,7 +473,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
 #if defined(USE_MYSQL_COOKIE)
     if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
       if (! (load_string = chxj_load_cookie_mysql(r, dconf, cookie->cookie_id))) {
-        ERR(r, "failed: chxj_load_cookie_mysql() cookie_id:[%s]", cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_load_cookie_mysql() cookie_id:[%s]", (apr_size_t)r, cookie_id);
         goto on_error0;
       }
       done_proc = 1;
@@ -483,7 +482,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
 #if defined(USE_MEMCACHE_COOKIE)
     if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
       if (! (load_string = chxj_load_cookie_memcache(r, dconf, cookie->cookie_id))) {
-        ERR(r, "failed: chxj_load_cookie_memcache() cookie_id:[%s]", cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_load_cookie_memcache() cookie_id:[%s]", (apr_size_t)r, cookie_id);
         goto on_error0;
       }
       done_proc = 1;
@@ -491,14 +490,14 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
 #endif
     if (!done_proc || IS_COOKIE_STORE_DBM(dconf->cookie_store_type)) {
       if (! (load_string = chxj_load_cookie_dbm(r, dconf, cookie->cookie_id))) {
-        ERR(r, "failed: chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+        ERR(r, "REQ[%X] failed: chxj_load_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
         goto on_error0;
       }
     }
   }
 
   if (load_string) {
-    DBG(r, "load_string=[%s]", load_string);
+    DBG(r, "REQ[%X] load_string=[%s]", (apr_size_t)r, load_string);
     header_cookie = apr_palloc(pool, 1);
     header_cookie[0] = 0;
     for (;;) {
@@ -507,7 +506,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
       load_string = NULL;
       if (!pair) break;
 
-      DBG(r, "Cookie:[%s]", pair);
+      DBG(r, "REQ[%X] Cookie:[%s]", (apr_size_t)r, pair);
       char* tmp_pair;
 
       tmp_pair = apr_pstrdup(pool, pair);
@@ -515,8 +514,13 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
       if (val) {
         key = tmp_pair;
         *val++ = 0;
-        apr_table_add(load_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
-        DBG(r, "ADD key:[%s] val:[%s]", key, val);
+        if (! apr_table_get(load_cookie_table, key)) {
+          apr_table_add(load_cookie_table, apr_pstrdup(pool, key), apr_pstrdup(pool, val));
+          DBG(r, "REQ[%X] ADD key:[%s] val:[%s]", (apr_size_t)r, key, val);
+        }
+        else {
+          continue;
+        }
       }
       tmp_pair = apr_pstrdup(pool, pair);
       tmp_sem = strchr(tmp_pair, ';'); 
@@ -531,7 +535,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
       }
     }
     if (strlen(header_cookie)) {
-      DBG(r, "ADD COOKIE to REQUEST HEADER:[%s]", header_cookie);
+      DBG(r, "REQ[%X] ADD COOKIE to REQUEST HEADER:[%s]", (apr_size_t)r, header_cookie);
       apr_table_add(r->headers_in, apr_pstrdup(pool, "Cookie"), apr_pstrdup(pool, header_cookie));
     }
   
@@ -549,22 +553,14 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
     apr_table_setn(r->headers_in, apr_pstrdup(pool, "CHXJ_COOKIE_ID"), apr_pstrdup(pool, cookie->cookie_id));
   }
 
-  DBG(r, "end   chxj_load_cookie()");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
+  DBG(r, "REQ[%X] end   chxj_load_cookie()",(apr_size_t)r);
 
   return cookie;
 
 
 on_error0:
 
-  DBG(r, "end   chxj_load_cookie()");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
-  DBG(r, "========================================================");
+  DBG(r, "REQ[%X] end   chxj_load_cookie()",(apr_size_t)r);
   return NULL;
 }
 
@@ -574,13 +570,13 @@ check_valid_cookie_attribute(request_rec *r, const char *value)
   char *pstat;
   char *pair;
   char *first_pair;
-  char *domain_pair;
-  char *path_pair;
-  char *expire_pair;
-  char *secure_pair;
+  char *domain_pair = NULL;
+  char *path_pair   = NULL;
+  char *expire_pair = NULL;
+  char *secure_pair = NULL;
   char *p;
 
-  DBG(r, "start check_valid_cookie_attribute() value:[%s]", value);
+  DBG(r, "REQ[%X] start check_valid_cookie_attribute() value:[%s]", (apr_size_t)r, value);
 
   domain_pair = path_pair = expire_pair = secure_pair = NULL;
   p = apr_pstrdup(r->pool, value);
@@ -608,29 +604,66 @@ check_valid_cookie_attribute(request_rec *r, const char *value)
 
   if (domain_pair) {
     if (!valid_domain(r, domain_pair)) {
-      DBG(r, "invalid domain. domain_pair:[%s]", domain_pair);
+      DBG(r, "REQ[%X] invalid domain. domain_pair:[%s]", (apr_size_t)r, domain_pair);
       return CHXJ_FALSE;
     }
   }
   if (path_pair) {
     if (!valid_path(r, path_pair)) {
-      DBG(r, "invalid path. path_pair:[%s]", path_pair);
+      DBG(r, "REQ[%X] invalid path. path_pair:[%s]", (apr_size_t)r, path_pair);
       return CHXJ_FALSE;
     }
   }
   if (expire_pair) {
     if (!valid_expires(r, expire_pair)) {
-      DBG(r, "invalid expire. expire_pair:[%s]", expire_pair);
+      DBG(r, "REQ[%X] invalid expire. expire_pair:[%s]", (apr_size_t)r, expire_pair);
       return CHXJ_FALSE;
     }
   }
   if (secure_pair) {
     if (!valid_secure(r, secure_pair)) {
-      DBG(r, "invalid secure. secure_pair:[%s]", secure_pair);
+      DBG(r, "REQ[%X] invalid secure. secure_pair:[%s]", (apr_size_t)r, secure_pair);
       return CHXJ_FALSE;
     }
   }
-  DBG(r, "end check_valid_cookie_attribute() value:[%s]", value);
+  DBG(r, "REQ[%X] end check_valid_cookie_attribute() value:[%s]", (apr_size_t)r, value);
+  return CHXJ_TRUE;
+}
+
+
+static int
+check_valid_cookie_attribute_expires_only(request_rec *r, const char *value)
+{
+  char *pstat;
+  char *pair;
+  char *first_pair;
+  char *expire_pair = NULL;
+  char *p;
+
+  DBG(r, "REQ[%X] start check_valid_cookie_attribute_expires_only() value:[%s]", (apr_size_t)r, value);
+
+  expire_pair = NULL;
+  p = apr_pstrdup(r->pool, value);
+
+  /* pass first pair */
+  first_pair = apr_strtok(p, ";", &pstat);
+
+  for (;;) {
+    pair = apr_strtok(NULL, ";", &pstat);
+    if (! pair) break;
+    pair = qs_trim_string(r->pool, pair);
+    if (STRNCASEEQ('e','E',"expires", pair, sizeof("expires")-1)) {
+      expire_pair = apr_pstrdup(r->pool, pair);
+    }
+  }
+
+  if (expire_pair) {
+    if (!valid_expires(r, expire_pair)) {
+      DBG(r, "REQ[%X] invalid expire. expire_pair:[%s]", (apr_size_t)r, expire_pair);
+      return CHXJ_FALSE;
+    }
+  }
+  DBG(r, "REQ[%X] end check_valid_cookie_attribute_expires_only() value:[%s]", (apr_size_t)r, value);
   return CHXJ_TRUE;
 }
 
@@ -644,23 +677,30 @@ valid_domain(request_rec *r, const char *value)
   char *p = apr_pstrdup(r->pool, value);
   const char *host = apr_table_get(r->headers_in, HTTP_HOST);
 
-  DBG(r, "start valid_domain() value:[%s]", value);
-  DBG(r, "host:[%s]", host);
-  if (!host)
+  DBG(r, "REQ[%X] start valid_domain() value:[%s]", (apr_size_t)r, value);
+  if (!host) {
+    DBG(r, "REQ[%X] end valid_domain() value:[%s] (has no host header)", (apr_size_t)r, value);
     return CHXJ_TRUE;
+  }
 
   name = apr_strtok(p,"=", &pstat);
   name = qs_trim_string(r->pool, name);
   val = apr_strtok(NULL, "=", &pstat);
   val = qs_trim_string(r->pool, val);
+
+  host = apr_pstrdup(r->pool, host);
+  char *tmp = strchr(host, ':');
+  if (tmp) {
+    *tmp = 0;
+  }
   len = strlen(host);
   if (len) {
     if (chxj_strcasenrcmp(r->pool, host, val, strlen(val))) {
-      DBG(r, "not match domain. host domain:[%s] vs value:[%s]", host, val);
+      DBG(r, "REQ[%X] not match domain. host domain:[%s] vs value:[%s]", (apr_size_t)r, host, val);
       return CHXJ_FALSE;
     }
   }
-  DBG(r, "end valid_domain() value:[%s]", value);
+  DBG(r, "REQ[%X] end valid_domain() value:[%s]", (apr_size_t)r, value);
   return CHXJ_TRUE;
 }
 
@@ -674,7 +714,7 @@ valid_path(request_rec *r, const char *value)
   char *val;
   char *pstat;
 
-  DBG(r, "start valid_path() unparsed_uri:[%s] value:[%s]", r->unparsed_uri, value);
+  DBG(r, "REQ[%X] start valid_path() unparsed_uri:[%s] value:[%s]", (apr_size_t)r, r->unparsed_uri, value);
   if (chxj_starts_with(r->unparsed_uri, "http://")) {
     uri = strchr(&r->unparsed_uri[sizeof("http://")], '/');
     if (uri != NULL) {
@@ -697,19 +737,18 @@ valid_path(request_rec *r, const char *value)
   if ((tmp = strchr(uri, '?'))) {
     *tmp = '\0';
   }
-  DBG(r, "uri=[%s]", uri);
+  DBG(r, "REQ[%X] uri=[%s]", (apr_size_t)r, uri);
   name = apr_strtok(p, "=", &pstat);
   val = apr_strtok(NULL, "=", &pstat);
   name = qs_trim_string(r->pool, name);
   val = qs_trim_string(r->pool, val);
   DBG(r, "name=[%s] val=[%s]", name, val);
   
-  DBG(r, "val:[%s] vs uri:[%s]", val, uri);
   if (! chxj_starts_with(uri, val)) {
-    DBG(r, "end valid_path() unparsed_uri:[%s] value:[%s] (false)", r->unparsed_uri, value);
+    DBG(r, "REQ[%X] end valid_path() unparsed_uri:[%s] value:[%s] (false)", (apr_size_t)r, r->unparsed_uri, value);
     return CHXJ_FALSE;
   }
-  DBG(r, "end valid_path() unparsed_uri:[%s] value:[%s] (true)", r->unparsed_uri, value);
+  DBG(r, "REQ[%X] end valid_path() unparsed_uri:[%s] value:[%s] (true)", (apr_size_t)r, r->unparsed_uri, value);
   return CHXJ_TRUE;
 }
 
@@ -722,7 +761,7 @@ valid_expires(request_rec *r, const char *value)
   char *pstat;
   apr_time_t expires;
   apr_time_t now;
-  DBG(r, "start valid_expire() value:[%s]", value);
+  DBG(r, "REQ[%X] start valid_expire() value:[%s]", (apr_size_t)r, value);
 
   name = apr_strtok(p, "=", &pstat);
   val  = apr_strtok(NULL, "=", &pstat);
@@ -730,11 +769,11 @@ valid_expires(request_rec *r, const char *value)
   now = apr_time_now();
   expires = chxj_parse_cookie_expires(val);
   if (expires < now) {
-    DBG(r, "end valid_expire() value:[%s] (expired)", value);
+    DBG(r, "REQ[%X] end valid_expire() value:[%s] (expired)", (apr_size_t)r, value);
     return CHXJ_FALSE;
   }
   
-  DBG(r, "end valid_expire() value:[%s] (non expired)", value);
+  DBG(r, "REQ[%X] end valid_expire() value:[%s] (non expired)", (apr_size_t)r, value);
   return CHXJ_TRUE;
 }
 
@@ -742,13 +781,13 @@ static int
 valid_secure(request_rec *r, const char *value)
 {
   const char *scheme;
-  DBG(r, "start valid_secure() value:[%s]", value);
+  DBG(r, "REQ[%X] start valid_secure() value:[%s]", (apr_size_t)r, value);
   scheme = chxj_apache_run_http_scheme(r);
   if (strcasecmp("https", scheme)) {
-    DBG(r, "end valid_secure() value:[%s] (non secure)", value);
+    DBG(r, "REQ[%X] end valid_secure() value:[%s] (non secure)", (apr_size_t)r, value);
     return CHXJ_FALSE;
   }
-  DBG(r, "end valid_secure() value:[%s] (secure)", value);
+  DBG(r, "REQ[%X] end valid_secure() value:[%s] (secure)", (apr_size_t)r, value);
   return CHXJ_TRUE;
 }
 
@@ -760,7 +799,7 @@ chxj_add_cookie_parameter(request_rec *r, char *value, cookie_t *cookie)
   char *dst;
   char *name = "";
 
-  DBG(r, "start chxj_add_cookie_parameter() cookie_id=[%s]", (cookie) ? cookie->cookie_id : NULL);
+  DBG(r, "REQ[%X] start chxj_add_cookie_parameter() cookie_id=[%s]", (apr_size_t)r, (cookie) ? cookie->cookie_id : NULL);
 
   dst = apr_pstrdup(r->pool, value);
 
@@ -771,7 +810,7 @@ chxj_add_cookie_parameter(request_rec *r, char *value, cookie_t *cookie)
     goto on_error;
 
   if (chxj_cookie_check_host(r, value) != 0) {
-    DBG(r, "end chxj_add_cookie_parameter()(check host)");
+    DBG(r, "REQ[%X] end chxj_add_cookie_parameter()(check host)", (apr_size_t)r);
     goto on_error;
   }
 
@@ -789,12 +828,11 @@ chxj_add_cookie_parameter(request_rec *r, char *value, cookie_t *cookie)
     dst = apr_psprintf(r->pool, "%s?%s=%s%s", dst, CHXJ_COOKIE_PARAM, cookie->cookie_id, name);
   }
 
-  DBG(r, "end   chxj_add_cookie_parameter() dst=[%s]", dst);
+  DBG(r, "REQ[%X] end   chxj_add_cookie_parameter() dst=[%s]", (apr_size_t)r, dst);
 
   return dst;
 
 on_error:
-  DBG(r, "end   chxj_add_cookie_parameter() (on_error)");
   return dst;
 }
 
@@ -804,15 +842,21 @@ chxj_cookie_check_host(request_rec* r, char* value)
 {
   char* hostnm;
 
-  DBG(r, "hostname=[%s]", r->hostname);
+  DBG(r, "REQ[%X] start chxj_cookie_check_host()", (apr_size_t)r);
+  DBG(r, "hostname=[%s] vs Location:[%s]", r->hostname, value);
 
   hostnm = s_get_hostname_from_url(r, value);
   if (hostnm) {
-    if (strcasecmp(hostnm, r->hostname) == 0)
+    if (strcasecmp(hostnm, r->hostname) == 0) {
+      DBG(r, "REQ[%X] end chxj_cookie_check_host() (true)", (apr_size_t)r);
       return 0;
-    else
+    }
+    else {
+      DBG(r, "REQ[%X] end chxj_cookie_check_host() (false)", (apr_size_t)r);
       return 1;
+    }
   }
+  DBG(r, "REQ[%X] end chxj_cookie_check_host() (true)", (apr_size_t)r);
   return 0;
 }
 
@@ -841,7 +885,7 @@ s_cut_until_end_hostname(request_rec* r, char* value)
 
   hostnm = sp = apr_pstrdup(r->pool, value);
   for (;*sp; sp++) {
-    if (*sp == '/'|| *sp == '?') {
+    if (*sp == '/'|| *sp == '?' || *sp == ':') {
       *sp = '\0';
       break;
     }
@@ -852,19 +896,19 @@ s_cut_until_end_hostname(request_rec* r, char* value)
 
 
 void
-chxj_delete_cookie(request_rec *r, const char *cookie_id)
+__chxj_delete_cookie(request_rec *r, const char *cookie_id, const char *__filename__, int __line__)
 {
   int done_proc = 0;
   mod_chxj_config *dconf;
 
-  DBG(r, "start chxj_delete_cookie()");
+  DBG(r, "REQ[%X] start chxj_delete_cookie() call from %s:%d", (apr_size_t)r, __filename__,__line__);
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
 #if defined(USE_MYSQL_COOKIE)
   if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
     if (! chxj_delete_cookie_mysql(r, dconf, cookie_id)) {
-      ERR(r, "failed: chxj_delete_cookie_mysql() cookie_id:[%s]", cookie_id);
-      DBG(r, "end   chxj_delete_cookie()");
+      ERR(r, "REQ[%X] failed: chxj_delete_cookie_mysql() cookie_id:[%s]", (apr_size_t)r, cookie_id);
+      DBG(r, "REQ[%X] end   chxj_delete_cookie() call from %s:%d ",(apr_size_t)r, __filename__,__line__);
       return;
     }
     done_proc = 1;
@@ -874,8 +918,8 @@ chxj_delete_cookie(request_rec *r, const char *cookie_id)
 #if defined(USE_MEMCACHE_COOKIE)
   if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
     if (! chxj_delete_cookie_memcache(r, dconf, cookie_id)) {
-      ERR(r, "failed: chxj_delete_cookie_memcache() cookie_id:[%s]", cookie_id);
-      DBG(r, "end   chxj_delete_cookie()");
+      ERR(r, "REQ[%X] failed: chxj_delete_cookie_memcache() cookie_id:[%s]", (apr_size_t)r, cookie_id);
+      DBG(r, "REQ[%X] end   chxj_delete_cookie() call from %s:%d",(apr_size_t)r,__filename__,__line__);
       return;
     }
     done_proc = 1;
@@ -883,13 +927,13 @@ chxj_delete_cookie(request_rec *r, const char *cookie_id)
 #endif
   if (!done_proc || IS_COOKIE_STORE_DBM(dconf->cookie_store_type)) {
     if (! chxj_delete_cookie_dbm(r, dconf, cookie_id)) {
-      ERR(r, "failed: chxj_delete_cookie_dbm() cookie_id:[%s]", cookie_id);
-      DBG(r, "end   chxj_delete_cookie()");
+      ERR(r, "REQ[%X] failed: chxj_delete_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
+      DBG(r, "REQ[%X] end   chxj_delete_cookie() call from %s:%d",(apr_size_t)r,__filename__,__line__);
       return;
     }
   }
 
-  DBG(r, "end   chxj_delete_cookie()");
+  DBG(r, "REQ[%X] end   chxj_delete_cookie() call from %s:%d",(apr_size_t)r, __filename__,__line__);
 }
 
 
@@ -1036,6 +1080,106 @@ chxj_parse_cookie_expires(const char *s)
   if (!s) return (apr_time_t)0;
   return apr_date_parse_rfc(s);
 }
+
+
+cookie_lock_t *
+__chxj_cookie_lock(request_rec *r, const char *filename, int line)
+{
+  mod_chxj_config *dconf;
+  apr_status_t rv;
+  int done_proc = 0;
+  cookie_lock_t *ret = NULL;
+
+  DBG(r, "start chxj_cookie_lock() call from %s:%d", filename, line);
+  if ((rv = apr_proc_mutex_lock(global_cookie_mutex)) != APR_SUCCESS) {
+    char errstr[255];
+    ERR(r, "%s:%d apr_proc_mutex_lock failure.(%d:%s)", APLOG_MARK, rv, apr_strerror(rv, errstr, 255));
+    return NULL;
+  }
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
+#if defined(USE_MYSQL_COOKIE)
+  if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
+    if (! chxj_cookie_lock_mysql(r, dconf)) {
+      ERR(r, "%s:%d end chxj_cookie_lock(): failed: chxj_cookie_lock_mysql()", APLOG_MARK);
+      return NULL;
+    }
+    done_proc = 1;
+    ret = apr_palloc(r->pool, sizeof(*ret));
+    memset(ret, 0, sizeof(*ret));
+  }
+#endif
+#if defined(USE_MEMCACHE_COOKIE)
+  if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
+    if (! chxj_cookie_lock_memcache(r, dconf)) {
+      ERR(r, "%s:%d end chxj_cookie_lock(): failed: chxj_cookie_lock_memcache()", APLOG_MARK);
+      return NULL;
+    }
+    done_proc = 1;
+    ret = apr_palloc(r->pool, sizeof(*ret));
+    memset(ret, 0, sizeof(*ret));
+  }
+#endif
+  if (!done_proc) {
+    if (!(ret = chxj_cookie_lock_dbm(r, dconf))) {
+      ERR(r, "%s:%d end chxj_cookie_lock(): failed: chxj_cookie_lock_dbm()", APLOG_MARK);
+      DBG(r, "end chxj_cookie_lock() call from %s:%d", filename, line);
+      return NULL;
+    }
+  }
+  DBG(r, "end chxj_cookie_lock() call from %s:%d", filename, line);
+  return ret;
+}
+
+
+int
+__chxj_cookie_unlock(request_rec *r, cookie_lock_t *lock, const char *filename, int line)
+{
+  mod_chxj_config *dconf;
+  int done_proc = 0;
+  apr_status_t rv;
+  int rtn = 1;
+
+  DBG(r, "start chxj_cookie_unlock() call from %s:%d", filename, line);
+
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
+#if defined(USE_MYSQL_COOKIE)
+  if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
+    if (! chxj_cookie_unlock_mysql(r, dconf)) {
+      ERR(r, "failed: chxj_cookie_unlock_mysql()");
+      rtn = 0;
+      goto end_chxj_cookie_unlock;
+    }
+    done_proc = 1;
+  }
+#endif
+#if defined(USE_MEMCACHE_COOKIE)
+  if (IS_COOKIE_STORE_MEMCACHE(dconf->cookie_store_type)) {
+    if (! chxj_cookie_unlock_memcache(r, dconf)) {
+      ERR(r, "failed: chxj_cookie_unlock_memcache()");
+      rtn = 0;
+      goto end_chxj_cookie_unlock;
+    }
+    done_proc = 1;
+  }
+#endif
+  if (!done_proc) {
+    if (! chxj_cookie_unlock_dbm(r, lock, dconf)) {
+      ERR(r, "failed: chxj_cookie_unlock_dbm()");
+      rtn = 0;
+      goto end_chxj_cookie_unlock;
+    }
+  }
+end_chxj_cookie_unlock:
+  if ((rv = apr_proc_mutex_unlock(global_cookie_mutex)) != APR_SUCCESS) {
+    char errstr[255];
+    ERR(r, "%s:%d apr_proc_mutex_unlock failure.(%d:%s)", APLOG_MARK, rv, apr_strerror(rv, errstr, 255));
+    DBG(r, "end chxj_cookie_unlock() call from %s:%d", filename, line);
+    return 0;
+  }
+  DBG(r, "end chxj_cookie_unlock() call from %s:%d", filename, line);
+
+  return rtn;
+}
 /*
  * vim:ts=2 et
  */
