@@ -40,12 +40,12 @@ chxj_save_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id,
   apr_dbm_t           *f;
   apr_file_t          *file;
 
-  DBG(r, "start chxj_save_cookie_dbm() cookie_id:[%s]", cookie_id);
+  DBG(r, "REQ[%X] start chxj_save_cookie_dbm() cookie_id:[%s] store_string:[%s]", (apr_size_t)r, cookie_id, store_string);
 
   file = chxj_cookie_db_lock(r);
   if (! file) {
     ERR(r, "mod_chxj: Can't lock cookie db");
-    DBG(r, "end chxj_save_cookie_dbm() cookie_id:[%s]", cookie_id);
+    DBG(r, "REQ[%X] end chxj_save_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
     return CHXJ_FALSE;
   }
 
@@ -60,7 +60,7 @@ chxj_save_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id,
             "default",
             chxj_cookie_db_name_create(r,m->cookie_db_dir));
     chxj_cookie_db_unlock(r, file);
-    DBG(r, "end chxj_save_cookie_dbm() cookie_id:[%s]", cookie_id);
+    DBG(r, "REQ[%X] end chxj_save_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
     return CHXJ_FALSE;
   }
 
@@ -83,13 +83,13 @@ chxj_save_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id,
             chxj_cookie_db_name_create(r, m->cookie_db_dir));
     apr_dbm_close(f);
     chxj_cookie_db_unlock(r, file);
-    DBG(r, "end chxj_save_cookie_dbm() cookie_id:[%s]", cookie_id);
+    DBG(r, "REQ[%X] end chxj_save_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
     return CHXJ_FALSE;
   }
 
   apr_dbm_close(f);
   chxj_cookie_db_unlock(r, file);
-  DBG(r, "end chxj_save_cookie_dbm() cookie_id:[%s]", cookie_id);
+  DBG(r, "REQ[%X] end chxj_save_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
   return CHXJ_TRUE;
 }
 
@@ -116,7 +116,7 @@ chxj_cookie_db_lock(request_rec *r)
   apr_status_t     rv;
   mod_chxj_config  *dconf;
 
-  dconf = (mod_chxj_config*)ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = (mod_chxj_config*)chxj_get_module_config(r->per_dir_config, &chxj_module);
 
   rv = apr_file_open(&file,
                      chxj_cookie_db_lock_name_create(r, dconf->cookie_db_dir),
@@ -242,7 +242,7 @@ chxj_cookie_expire_db_lock(request_rec *r)
   apr_status_t     rv;
   mod_chxj_config  *dconf;
 
-  dconf = (mod_chxj_config *)ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = (mod_chxj_config *)chxj_get_module_config(r->per_dir_config, &chxj_module);
 
   rv = apr_file_open(&file, 
                      chxj_cookie_expire_db_lock_name_create(r, dconf->cookie_db_dir),
@@ -355,11 +355,11 @@ chxj_load_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id)
   apr_datum_t             dbmval;
   apr_datum_t             dbmkey;
 
-  DBG(r, "start chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+  DBG(r, "REQ[%X] start chxj_load_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
   file = chxj_cookie_db_lock(r);
   if (! file) {
     ERR(r, "mod_chxj: Can't lock cookie db");
-    DBG(r, "end   chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+    DBG(r, "REQ[%X] end   chxj_load_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
     return NULL;
   }
 
@@ -371,11 +371,13 @@ chxj_load_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id)
                            r->pool);
   if (retval != APR_SUCCESS) {
     ERR(r,
+         "%s:%d"
          "could not open dbm (type %s) auth file: %s",
+         __FILE__,__LINE__,
          "default",
          chxj_cookie_db_name_create(r, m->cookie_db_dir));
     chxj_cookie_db_unlock(r, file);
-    DBG(r, "end   chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+    DBG(r, "REQ[%X] end   chxj_load_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
     return NULL;
   }
 
@@ -389,11 +391,14 @@ chxj_load_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id)
     retval = apr_dbm_fetch(f, dbmkey, &dbmval);
     if (retval != APR_SUCCESS) {
       ERR(r,
-           "could not fetch dbm (type %s) auth file: %s", "default",
+           "%s:%d"
+           "could not fetch dbm (type %s) auth file: %s",
+           __FILE__,__LINE__,
+           "default",
            chxj_cookie_db_name_create(r, m->cookie_db_dir));
       apr_dbm_close(f);
       chxj_cookie_db_unlock(r, file);
-      DBG(r, "end   chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+      DBG(r, "REQ[%X] end   chxj_load_cookie_dbm() cookie_id:[%s]", (apr_size_t)r, cookie_id);
       return NULL;
     }
     load_string = apr_palloc(r->pool, dbmval.dsize+1);
@@ -403,7 +408,7 @@ chxj_load_cookie_dbm(request_rec *r, mod_chxj_config *m, const char *cookie_id)
   }
   apr_dbm_close(f);
   chxj_cookie_db_unlock(r, file);
-  DBG(r, "end   chxj_load_cookie_dbm() cookie_id:[%s]", cookie_id);
+  DBG(r, "REQ[%X] end   chxj_load_cookie_dbm() cookie_id:[%s] load_string:[%s]", (apr_size_t)r, cookie_id, load_string);
   return load_string;
 }
 
@@ -662,6 +667,23 @@ chxj_cookie_expire_gc_dbm(request_rec *r, mod_chxj_config *m)
   DBG(r, "end   chxj_cookie_expire_gc_dbm()");
   return CHXJ_TRUE;
 }
+
+
+cookie_lock_t *
+chxj_cookie_lock_dbm(request_rec *r, mod_chxj_config *UNUSED(m))
+{
+  cookie_lock_t *ret = apr_palloc(r->pool, sizeof(*ret));
+  ret->file = chxj_cookie_db_lock(r);
+  return ret;
+}
+
+
+int
+chxj_cookie_unlock_dbm(request_rec *r, cookie_lock_t *lock, mod_chxj_config *UNUSED(m))
+{
+  chxj_cookie_expire_db_unlock(r, lock->file);
+  return 1; /* allways true */
+}
 /*
  * vim:ts=2 et
  */
