@@ -143,7 +143,12 @@ chxj_headers_fixup(request_rec *r)
   char                *contentType;
   char                *contentLength;
 
-  DBG(r, "start chxj_headers_fixup()");
+  DBG(r, "REQ[%X] start chxj_headers_fixup()", (apr_size_t)r);
+  if (r->main) {
+    DBG(r, "REQ[%X] detect internal redirect.", (apr_size_t)r);
+    DBG(r, "REQ[%X] end chxj_headers_fixup()", (apr_size_t)r);
+    return DECLINED;
+  }
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
   user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
@@ -152,8 +157,8 @@ chxj_headers_fixup(request_rec *r)
   contentType = (char *)apr_table_get(r->headers_in, "Content-Type");
   if (contentType
       && strncasecmp("multipart/form-data", contentType, 19) == 0) {
-    DBG(r, "detect multipart/form-data ==> no target");
-    DBG(r, "end chxj_headers_fixup()");
+    DBG(r, "REQ[%X] detect multipart/form-data ==> no target", (apr_size_t)r);
+    DBG(r, "REQ[%X] end chxj_headers_fixup()", (apr_size_t)r);
     return DECLINED;
   }
 
@@ -177,7 +182,7 @@ chxj_headers_fixup(request_rec *r)
   case CHXJ_SPEC_Jhtml:
     entryp = chxj_apply_convrule(r, dconf->convrules);
     if (! entryp) {
-      DBG(r, "end chxj_headers_fixup() no pattern");
+      DBG(r, "REQ[%X] end chxj_headers_fixup() no pattern User-Agent:[%s]", (apr_size_t)r, user_agent);
       return DECLINED;
     }
     if (!entryp || !(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
@@ -314,7 +319,11 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
     case CHXJ_SPEC_Chtml_5_0:
     case CHXJ_SPEC_XHtml_Mobile_1_0:
     case CHXJ_SPEC_Jhtml:
+      {
+      cookie_lock_t *lock = chxj_cookie_lock(r);
       cookie = chxj_save_cookie(r);
+      chxj_cookie_unlock(r, lock);
+      }
       break;
     default:
       break;
@@ -396,7 +405,6 @@ s_clear_cookie_header(request_rec *r, device_table *spec)
 static int
 chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp) 
 {
-
   char*      buff;
   char*      buff_pre;
   apr_size_t urilen;
@@ -406,14 +414,13 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp)
   char*      value;
   char*      pstate;
   char*      vstate;
-  cookie_t*  cookie;
+  cookie_t   *cookie = NULL;
   int        no_update_flag = 0;
 
-  DBG(r, "start chxj_convert_input_header()");
-
+  DBG(r, "REQ[%X] start chxj_convert_input_header()", (apr_size_t)r);
   if (! r->args) {
-    DBG(r, "r->args=[null]");
-    DBG(r, "end   chxj_convert_input_header()");
+    DBG(r, "REQ[%X] r->args=[null]", (apr_size_t)r);
+    DBG(r, "REQ[%X] end   chxj_convert_input_header()", (apr_size_t)r);
     return 0;
   }
   urilen = strlen(r->args);
@@ -435,14 +442,15 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp)
 
     name  = apr_strtok(pair, "=", &vstate);
     value = apr_strtok(NULL, "=", &vstate);
+    if (! name) continue;
     if (strcasecmp(name, CHXJ_COOKIE_NOUPDATE_PARAM) == 0 || strcasecmp(name, chxj_url_encode(r->pool, CHXJ_COOKIE_NOUPDATE_PARAM)) == 0) {
-      DBG(r, "found cookie no update parameter");
+      DBG(r, "REQ[%X] found cookie no update parameter", (apr_size_t)r);
       no_update_flag++;
     }
   }
 
   buff = apr_pstrdup(r->pool, r->args);
-  DBG(r, "r->args=[%s]", buff);
+  DBG(r, "REQ[%X] r->args=[%s]", (apr_size_t)r, buff);
 
   /* _chxj_dmy */
   /* _chxj_c_ */
@@ -461,6 +469,7 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp)
 
     name  = apr_strtok(pair, "=", &vstate);
     value = apr_strtok(NULL, "=", &vstate);
+    if (!name) continue;
     if (strncasecmp(name, "_chxj", 5) != 0 && strncasecmp(name, "%5Fchxj", sizeof("%5Fchxj")-1) != 0) {
       if (strlen(result) != 0) 
         result = apr_pstrcat(r->pool, result, "&", NULL);
@@ -473,12 +482,8 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp)
         if (value && *value != 0) {
           value = chxj_url_decode(r->pool, value);
           dlen   = strlen(value);
-          DBG(r, "************ before encoding[%s]", value);
-  
           dvalue = chxj_rencoding(r, value, &dlen);
           dvalue = chxj_url_encode(r->pool, dvalue);
-  
-          DBG(r, "************ after encoding[%s]", dvalue);
         }
         else {
           dvalue = "";
@@ -504,37 +509,61 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry* entryp)
       }
     }
     else
-    if ( (strncasecmp(name, "_chxj_c_", 8) == 0 || strncasecmp(name, "%5Fchxj%5Fc%5F", sizeof("%5Fchxj%5Fc%5F")-1) == 0)
-      || (strncasecmp(name, "_chxj_r_", 8) == 0 || strncasecmp(name, "%5Fchxj%5Fr%5F", sizeof("%5Fchxj%5Fr%5F")-1) == 0)
-      || (strncasecmp(name, "_chxj_s_", 8) == 0 || strncasecmp(name, "%5Fchxj%5Fs%5F", sizeof("%5Fchxj%5Fs%5F")-1) == 0)) {
-      if (value == NULL)
-        continue;
-
-      if (strlen(value) == 0)
-        continue;
-
+    if ( strncasecmp(name, "_chxj_c_", 8) == 0
+      || strncasecmp(name, "_chxj_r_", 8) == 0
+      || strncasecmp(name, "_chxj_s_", 8) == 0) {
+      if (value == NULL) continue;
+      if (strlen(value) == 0) continue;
       if (strlen(result) != 0)
         result = apr_pstrcat(r->pool, result, "&", NULL);
-
       result = apr_pstrcat(r->pool, result, &name[8], "=", value, NULL);
     }
     else
+    if ( strncasecmp(name, "%5Fchxj%5Fr%5F", 14) == 0
+      || strncasecmp(name, "%5Fchxj%5Fc%5F", 14) == 0
+      || strncasecmp(name, "%5Fchxj%5Fs%5F", 14) == 0) {
+      if (value == NULL) continue;
+      if (strlen(value) == 0) continue;
+      if (strlen(result) != 0)
+        result = apr_pstrcat(r->pool, result, "&", NULL);
+      result = apr_pstrcat(r->pool, result, &name[14], "=", value, NULL);
+    }
+    else
     if (strcasecmp(name, CHXJ_COOKIE_PARAM) == 0 || strcasecmp(name, "%5Fchxj%5Fcc") == 0) {
-      apr_table_unset(r->headers_in, "Cookie");
-      DBG(r, "found cookie parameter[%s]", value);
-      DBG(r, "call start chxj_load_cookie()");
-      cookie = chxj_load_cookie(r, value);
-      DBG(r, "call end   chxj_load_cookie()");
-      if (! no_update_flag && cookie) {
-        chxj_update_cookie(r, cookie);
+      if (! cookie) {
+        apr_table_unset(r->headers_in, "Cookie");
+        cookie_lock_t *lock = chxj_cookie_lock(r);
+        DBG(r, "REQ[%X] found cookie parameter[%s]", (apr_size_t)r, value);
+        DBG(r, "REQ[%X] call start chxj_load_cookie()",(apr_size_t)r);
+        cookie = chxj_load_cookie(r, value);
+        DBG(r, "REQ[%X] call end   chxj_load_cookie()",(apr_size_t)r);
+        if (! no_update_flag && cookie) {
+          cookie = chxj_update_cookie(r, cookie);
+        }
+        chxj_cookie_unlock(r, lock);
       }
+      if (cookie && cookie->cookie_id) {
+        if (strlen(result) != 0)
+          result = apr_pstrcat(r->pool, result, "&", NULL);
+        result = apr_pstrcat(r->pool, result, name, "=", cookie->cookie_id, NULL);
+      }
+    }
+    else
+    if (strcasecmp(name, CHXJ_COOKIE_NOUPDATE_PARAM) == 0) {
+      if (strlen(result) != 0)
+        result = apr_pstrcat(r->pool, result, "&", NULL);
+      result = apr_pstrcat(r->pool, result, name, "=", value, NULL);
     }
     DBG(r, "************************ name:[%s]", name);
   }
+  apr_table_setn(r->headers_in, "X-Chxj-Cookie-No-Update", "true");
+  if (! no_update_flag) {
+    result = apr_pstrcat(r->pool, result, "&_chxj_nc=true", NULL);
+  }
   r->args = result;
 
-  DBG(r, "result r->args=[%s]", r->args);
-  DBG(r, "end   chxj_convert_input_header()");
+  DBG(r, "REQ[%X] result r->args=[%s]", (apr_size_t)r, r->args);
+  DBG(r, "REQ[%X] end   chxj_convert_input_header()",(apr_size_t)r);
   return 0;
 }
 
@@ -586,6 +615,7 @@ chxj_input_convert(
 
     name  = apr_strtok(pair, "=", &vstate);
     value = apr_strtok(NULL, "=", &vstate);
+    if (!name) continue;
     if (strcasecmp(name, CHXJ_COOKIE_NOUPDATE_PARAM) == 0) {
       DBG(r, "found cookie no update parameter");
       no_update_flag++;
@@ -604,6 +634,7 @@ chxj_input_convert(
 
     name  = apr_strtok(pair, "=", &vstate);
     value = apr_strtok(NULL, "=", &vstate);
+    if (!name) continue;
     if (strncasecmp(name, "_chxj", 5) != 0 && strncasecmp(name, "%5Fchxj", sizeof("%5Fchxj")-1) != 0) {
       if (strlen(result) != 0) 
         result = apr_pstrcat(r->pool, result, "&", NULL);
@@ -643,9 +674,9 @@ chxj_input_convert(
         result = apr_pstrcat(r->pool, result, name, "=", value, NULL);
       }
     }
-    else if ((strncasecmp(name, "_chxj_c_", 8) == 0  || strncasecmp(name, "%5Fchxj%5Fc%5F", sizeof("%5Fchxj%5Fc%5F")-1) == 0)
-        ||   (strncasecmp(name, "_chxj_r_", 8) == 0  || strncasecmp(name, "%5Fchxj%5Fr%5F", sizeof("%5Fchxj%5Fr%5F")-1) == 0)
-        ||   (strncasecmp(name, "_chxj_s_", 8) == 0  || strncasecmp(name, "%5Fchxj%5Fs%5F", sizeof("%5Fchxj%5Fs%5F")-1) == 0)) {
+    else if (strncasecmp(name, "_chxj_c_", 8) == 0
+        ||   strncasecmp(name, "_chxj_r_", 8) == 0
+        ||   strncasecmp(name, "_chxj_s_", 8) == 0) {
       if (value == NULL)
         continue;
 
@@ -661,37 +692,108 @@ chxj_input_convert(
 
         dlen   = strlen(value);
         value = chxj_url_decode(r->pool, value);
-        DBG(r, "************ before encoding[%s]", value);
-
         dvalue = chxj_rencoding(r, value, &dlen);
         dvalue = chxj_url_encode(r->pool,dvalue);
-
-        DBG(r, "************ after encoding[%s]", dvalue);
-
         result = apr_pstrcat(r->pool, result, &name[8], "=", dvalue, NULL);
-
       }
       else {
         result = apr_pstrcat(r->pool, result, &name[8], "=", value, NULL);
       }
     }
     else
-    if (strcasecmp(name, CHXJ_COOKIE_PARAM) == 0 || strcasecmp(name, "%5Fchxj%5Fcc") == 0) {
-      apr_table_unset(r->headers_in, "Cookie");
-      DBG(r, "found cookie parameter[%s]", value);
-      DBG(r, "call start chxj_load_cookie()");
-      cookie = chxj_load_cookie(r, value);
-      DBG(r, "call end   chxj_load_cookie()");
-      if (! no_update_flag && cookie) {
-        chxj_update_cookie(r, cookie);
+    if ( strncasecmp(name, "%5Fchxj%5Fs%5F", 14) == 0
+      || strncasecmp(name, "%5Fchxj%5Fr%5F", 14) == 0
+      || strncasecmp(name, "%5Fchxj%5Fc%5F", 14) == 0) {
+      if (value == NULL)
+        continue;
+
+      if (strlen(value) == 0)
+        continue;
+
+      if (strlen(result) != 0)
+        result = apr_pstrcat(r->pool, result, "&", NULL);
+
+      if (strcasecmp(entryp->encoding, "NONE") != 0 && value && strlen(value)) {
+        apr_size_t dlen;
+        char*      dvalue;
+        dlen   = strlen(value);
+        value = chxj_url_decode(r->pool, value);
+        dvalue = chxj_rencoding(r, value, &dlen);
+        dvalue = chxj_url_encode(r->pool,dvalue);
+        result = apr_pstrcat(r->pool, result, &name[14], "=", dvalue, NULL);
+      }
+      else {
+        result = apr_pstrcat(r->pool, result, &name[14], "=", value, NULL);
       }
     }
-    DBG(r, "************************ name:[%s]", name);
+    else
+    if (strcasecmp(name, CHXJ_COOKIE_PARAM) == 0 || strcasecmp(name, "%5Fchxj%5Fcc") == 0) {
+      apr_table_unset(r->headers_in, "Cookie");
+      cookie_lock_t *lock = chxj_cookie_lock(r);
+      DBG(r, "REQ[%X] found cookie parameter[%s]", (apr_size_t)r, value);
+      DBG(r, "REQ[%X] call start chxj_load_cookie()",(apr_size_t)r);
+      cookie = chxj_load_cookie(r, value);
+      DBG(r, "REQ[%X] call end   chxj_load_cookie()",(apr_size_t)r);
+      if (! no_update_flag && cookie) {
+        cookie = chxj_update_cookie(r, cookie);
+      }
+      chxj_cookie_unlock(r, lock);
+      if (cookie && cookie->cookie_id) {
+        if (strlen(result) != 0)
+          result = apr_pstrcat(r->pool, result, "&", NULL);
+        result = apr_pstrcat(r->pool, result, name, "=", cookie->cookie_id, NULL);
+      }
+    }
+    else
+    if (strcasecmp(name, CHXJ_COOKIE_NOUPDATE_PARAM) == 0) {
+      if (strlen(result) != 0)
+        result = apr_pstrcat(r->pool, result, "&", NULL);
+      result = apr_pstrcat(r->pool, result, name, "=", value, NULL);
+    }
+    else
+    if ( strncasecmp(name, CHXJ_QUERY_STRING_PARAM_PREFIX,     sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX)-1) == 0) {
+      apr_size_t dlen;
+      char*      dvalue;
+      dlen   = strlen(value);
+      if (dlen && value) {
+        value = chxj_url_decode(r->pool, value);
+        dvalue = chxj_rencoding(r, value, &dlen);
+        dvalue = chxj_url_encode(r->pool,dvalue);
+        if (r->args && strlen(r->args) > 0) {
+          r->args = apr_pstrcat(r->pool, r->args, "&", &name[sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX)-1], "=", dvalue, NULL);
+        }
+        else {
+          r->args = apr_pstrcat(r->pool, &name[sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX)-1], "=", dvalue, NULL);
+        }
+      }
+    }
+    else
+    if (strncasecmp(name, CHXJ_QUERY_STRING_PARAM_PREFIX_ENC, sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX_ENC)-1) == 0) {
+      apr_size_t dlen;
+      char*      dvalue;
+      dlen   = strlen(value);
+      if (dlen && value) {
+        value = chxj_url_decode(r->pool, value);
+        dvalue = chxj_rencoding(r, value, &dlen);
+        dvalue = chxj_url_encode(r->pool,dvalue);
+        if (r->args && strlen(r->args) > 0) {
+          r->args = apr_pstrcat(r->pool, r->args, "&", &name[sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX_ENC)-1], "=", dvalue, NULL);
+        }
+        else {
+          r->args = apr_pstrcat(r->pool, &name[sizeof(CHXJ_QUERY_STRING_PARAM_PREFIX_ENC)-1], "=", dvalue, NULL);
+        }
+      }
+    }
+    DBG(r, "REQ[%X] ************************ name:[%s]", (apr_size_t)r, name);
   }
   *len = strlen(result);
+  apr_table_setn(r->headers_in, "X-Chxj-Cookie-No-Update", "true");
+  if (! no_update_flag) {
+    result = apr_pstrcat(r->pool, result, "&_chxj_nc=true", NULL);
+  }
 
-  DBG(r, "AFTER input convert result = [%s]", result);
-  DBG(r, "end chxj_input_convert()");
+  DBG(r, "REQ[%X] AFTER input convert result = [%s]", (apr_size_t)r, result);
+  DBG(r, "REQ[%X] end chxj_input_convert()", (apr_size_t)r);
 
   return result;
 }
@@ -805,7 +907,11 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
         case CHXJ_SPEC_Chtml_5_0:
         case CHXJ_SPEC_XHtml_Mobile_1_0:
         case CHXJ_SPEC_Jhtml:
+          {
+          cookie_lock_t *lock = chxj_cookie_lock(r);
           cookie = chxj_save_cookie(r);
+          chxj_cookie_unlock(r,lock);
+          }
           s_add_cookie_id_if_has_location_header(r, cookie);
           apr_table_unset(r->headers_out, "Set-Cookie");
           apr_table_unset(r->err_headers_out, "Set-Cookie");
@@ -1018,7 +1124,11 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
           case CHXJ_SPEC_Chtml_5_0:
           case CHXJ_SPEC_XHtml_Mobile_1_0:
           case CHXJ_SPEC_Jhtml:
+            {
+            cookie_lock_t *lock = chxj_cookie_lock(r);
             cookie = chxj_save_cookie(r);
+            chxj_cookie_unlock(r, lock);
+            }
             /*
              * Location Header Check to add cookie parameter.
              */
@@ -1063,6 +1173,7 @@ s_add_cookie_id_if_has_location_header(request_rec *r, cookie_t *cookie)
     location_header = chxj_add_cookie_parameter(r,
                                                 location_header,
                                                 cookie);
+    location_header = apr_pstrcat(r->pool, location_header, "&_chxj_nc=true");
     apr_table_unset(r->headers_out, "Location");
     apr_table_add(r->headers_out, apr_pstrdup(r->pool, "Location"), location_header);
     DBG(r, "Location Header=[%s]", location_header);
@@ -1089,10 +1200,10 @@ chxj_input_handler(request_rec *r)
   char                *user_agent;
   apr_pool_t          *pool;
   
-  DBG(r, "start of chxj_input_handler()");
+  DBG(r, "REQ[%X] start of chxj_input_handler()", (apr_size_t)r);
 
   if (strcasecmp(r->handler, "chxj-input-handler")) {
-    DBG(r, "end chxj_input_handler()");
+    DBG(r, "REQ[%X] end chxj_input_handler() (NO chxj-input-handler)", (apr_size_t)r);
     return DECLINED;
   }
   apr_pool_create(&pool, r->pool);
@@ -1125,7 +1236,8 @@ chxj_input_handler(request_rec *r)
    */
   if (post_data_len > 0) {
     post_data = chxj_input_convert(r, (const char**)&post_data, (apr_size_t*)&post_data_len, entryp);
-    DBG(r, "(in:exchange)POSTDATA:[%s]", post_data);
+    post_data = apr_pstrcat(r->pool, post_data, "&_chxj_nc=true", NULL);
+    DBG(r, "REQ[%X] (in:exchange)POSTDATA:[%s]", (apr_size_t)r, post_data);
   }
 
   char *url_path;
@@ -1138,14 +1250,14 @@ chxj_input_handler(request_rec *r)
   if (r->args) {
     url_path = apr_pstrcat(pool, url_path, "?", r->args, NULL);
   }
-  DBG(r, "==> new url_path:[%s]", url_path);
+  DBG(r, "REQ[%X] ==> new url_path:[%s]", (apr_size_t)r, url_path);
 
   apr_size_t res_len;
   apr_table_setn(r->headers_in, CHXJ_HEADER_ORIG_CLIENT_IP, r->connection->remote_ip);
   apr_table_unset(r->headers_in, "Content-Length");
   apr_table_setn(r->headers_in, "Content-Length", apr_psprintf(pool, "%d", post_data_len));
   response = chxj_serf_post(r, pool, url_path, post_data, post_data_len, 1, &res_len);
-  DBG(r, "response:[%.*s][%d]", res_len, response, res_len);
+  DBG(r, "REQ[%X] response:[%.*s][%d]", (apr_size_t)r, res_len, response, res_len);
 
   char *chunked;
   if ((chunked = (char *)apr_table_get(r->headers_out, "Transfer-Encoding")) != NULL) {
@@ -1153,7 +1265,7 @@ chxj_input_handler(request_rec *r)
       r->chunked = 1;  
       apr_table_unset(r->headers_out, "Transfer-Encoding");
     }
-    DBG(r, "TRANSFER-ENCODING:[%s]",chunked);
+    DBG(r, "REQ[%X] TRANSFER-ENCODING:[%s]",(apr_size_t)r, chunked);
   }
   {
     apr_pool_t *wpool;
@@ -1169,13 +1281,13 @@ chxj_input_handler(request_rec *r)
     e = apr_bucket_eos_create(c->bucket_alloc);
     APR_BRIGADE_INSERT_TAIL(bb, e);
     if ((rv = ap_pass_brigade(r->output_filters, bb)) != APR_SUCCESS) {
-      ERR(r, "%s:%d failed ap_pass_brigade()", APLOG_MARK);
+      ERR(r, "REQ[%X] %s:%d failed ap_pass_brigade()", (apr_size_t)r, APLOG_MARK);
       return rv;
     }
     apr_brigade_cleanup(bb);
   }  
 
-  DBG(r, "end of chxj_input_handler()");
+  DBG(r, "REQ[%X] end of chxj_input_handler()", (apr_size_t)r);
   return APR_SUCCESS;
 }
 
@@ -1210,6 +1322,7 @@ chxj_init_module(apr_pool_t *p,
                   server_rec *s)
 {
   void *user_data;
+  apr_status_t rv;
 
   SDBG(s, "start chxj_init_module()");
 
@@ -1231,6 +1344,12 @@ chxj_init_module(apr_pool_t *p,
 
   ap_add_version_component(p, CHXJ_VERSION_PREFIX CHXJ_VERSION);
 
+  if ((rv = apr_proc_mutex_create(&global_cookie_mutex, NULL,  APR_LOCK_FCNTL, s->process->pool)) != APR_SUCCESS) {
+    char errstr[255];
+    SERR(s, "%s:%d end chxj_init_module(). mutex create failure.(%d:%s)",APLOG_MARK, rv,apr_strerror(rv,errstr,255));
+    return HTTP_INTERNAL_SERVER_ERROR;
+  }
+
   SDBG(s, "end  chxj_init_module()");
 
   return OK;
@@ -1238,8 +1357,15 @@ chxj_init_module(apr_pool_t *p,
 
 
 static void 
-chxj_child_init(apr_pool_t* UNUSED(p), server_rec *UNUSED(s))
+chxj_child_init(apr_pool_t* UNUSED(p), server_rec *s)
 {
+  apr_status_t rv;
+  SDBG(s, "start chxj_child_init()");
+  if ((rv = apr_proc_mutex_child_init(&global_cookie_mutex, NULL, s->process->pool)) != APR_SUCCESS) {
+    char errstr[255];
+    SERR(s, "%s:%d ERROR end chxj_init_module(). mutex create failure.(%d:%s)", APLOG_MARK, rv,apr_strerror(rv,errstr,255));
+  }
+  SDBG(s, "end   chxj_child_init()");
 }
 
 
@@ -1263,6 +1389,11 @@ chxj_config_server_create(apr_pool_t *p, server_rec *s)
 static int
 chxj_translate_name(request_rec *r)
 {
+  DBG(r, " /*===========================================");
+  DBG(r, " ");
+  DBG(r, " REQ[%X] START REQUEST(uri:[%s])", (apr_size_t)r, r->unparsed_uri);
+  DBG(r, " ");
+  DBG(r, " *===========================================*/");
   return chxj_trans_name(r);
 }
 
